aboutsummaryrefslogtreecommitdiffstats
diff options
context:
space:
mode:
authorRomain Forlot <romain.forlot@iot.bzh>2017-03-06 19:16:30 +0100
committerRomain Forlot <romain.forlot@iot.bzh>2017-03-16 17:04:39 +0100
commit4fd376fac5b1d47eed3f22467ae5b5e35f1ec610 (patch)
tree4a38eb9db42c57dfb1d80f05e334a19d916c85fb
parent7578df6fb02f05aea7fb69938f8a15a9c3e6c53a (diff)
Raw import openXC function to detect obd2 response.
Change-Id: Ieea0396979302329f5884ebef2064bdc7015a45f Signed-off-by: Romain Forlot <romain.forlot@iot.bzh>
-rw-r--r--src/obd2-signals.cpp39
-rw-r--r--src/obd2-signals.hpp11
2 files changed, 46 insertions, 4 deletions
diff --git a/src/obd2-signals.cpp b/src/obd2-signals.cpp
index 69db5e2..1a562d0 100644
--- a/src/obd2-signals.cpp
+++ b/src/obd2-signals.cpp
@@ -102,6 +102,45 @@ void shims_timer()
{
}
+bool is_obd2_response(can_message_t can_message)
+{
+ if(can_message.get_id() >= 0x7E8 && can_message.get_id() <= 0x7EF)
+ {
+ openxc_VehicleMessage message = {0};
+ message.has_type = true;
+ message.type = openxc_VehicleMessage_Type_DIAGNOSTIC;
+ message.has_diagnostic_response = true;
+ message.diagnostic_response = {0};
+ message.diagnostic_response.has_bus = true;
+ message.diagnostic_response.bus = bus->address;
+ message.diagnostic_response.has_message_id = true;
+ //7DF should respond with a random message id between 7e8 and 7ef
+ //7E0 through 7E7 should respond with a id that is 8 higher (7E0->7E8)
+ if(commandRequest->message_id == 0x7DF)
+ {
+ message.diagnostic_response.message_id = rand()%(0x7EF-0x7E8 + 1) + 0x7E8;
+ }
+ else if(commandRequest->message_id >= 0x7E0 && commandRequest->message_id <= 0x7E7)
+ {
+ message.diagnostic_response.message_id = commandRequest->message_id + 8;
+ }
+ message.diagnostic_response.has_mode = true;
+ message.diagnostic_response.mode = commandRequest->mode;
+ if(commandRequest->has_pid)
+ {
+ message.diagnostic_response.has_pid = true;
+ message.diagnostic_response.pid = commandRequest->pid;
+ }
+ message.diagnostic_response.has_value = true;
+ message.diagnostic_response.value = rand() % 100;
+ pipeline::publish(&message, &getConfiguration()->pipeline);
+ }
+ else //If it's outside the range, the command_request will return false
+ {
+ debug("Sent message ID is outside the valid range for emulator (7DF to 7E7)");
+ status=false;
+};
+
/*
* Will scan for supported Obd2 pids
*
diff --git a/src/obd2-signals.hpp b/src/obd2-signals.hpp
index 5ff63f8..c9fa4f6 100644
--- a/src/obd2-signals.hpp
+++ b/src/obd2-signals.hpp
@@ -21,6 +21,7 @@
#include "uds/uds.h"
#include "can-bus.hpp"
+#include "can-message.hpp"
#include "low-can-binding.hpp"
@@ -64,6 +65,8 @@ std::vector<Obd2Pid>& get_obd2_signals();
uint32_t get_signal_id(const Obd2Pid& sig);
void find_obd2_signals(const openxc_DynamicField &key, std::vector<Obd2Pid*>& found_signals);
+bool is_obd2_response(can_message_t can_message);
+
/**
* @brief - Object to handle obd2 session with pre-scan of supported pid
* then request them regularly
@@ -83,14 +86,14 @@ class obd2_handler_t {
/**
* @brief Check if a request is an OBD-II PID request.
*
- * @return true if the request is a mode 1 request and it has a 1 byte PID.
+ * @return true if the request is a mode 1 request and it has a 1 byte PID.
*/
bool is_obd2_request(DiagnosticRequest *request);
/**
* @brief Check if requested signal name is an obd2 pid
*
* @return true if name began with ob2.* else false.
*/
bool is_obd2_signal(const char *name);
@@ -103,8 +106,8 @@ class obd2_handler_t {
* http://en.wikipedia.org/wiki/OBD-II_PIDs#Mode_01).
*
* @param[in] DiagnosticResponse response - the received DiagnosticResponse (the data is in response.payload,
- * a byte array). This is most often used when the byte order is
- * signiticant, i.e. with many OBD-II PID formulas.
+ * a byte array). This is most often used when the byte order is
+ * signiticant, i.e. with many OBD-II PID formulas.
* @param[in] float parsed_payload - the entire payload of the response parsed as an int.
*/
float handle_obd2_pid(const DiagnosticResponse* response, float parsedPayload);
@@ -83,14 +86,14 @@ class obd2_handler_t {
* @brief Check if requested signal name is an obd2 pid
*
- * @return true if name began with ob2.* else false.
+ * @return true if name began with obd2 else false.
*/
bool is_obd2_signal(const char *name);